@@ -34,8 +34,8 @@ contains
 !----------------------------------------------------
 subroutine adjust_data_codeunits
  use system_utils,  only:renvironment,envlist,ienvironment,lenvironment
- use labels,        only:ih,ix,ivx,label,get_sink_type,ipmass
- use settings_data, only:ncolumns,ndimV,icoords,ndim,debugmode,ntypes,iverbose
+ use labels,        only:ih,ix,ivx,label,get_sink_type,ipmass,idustfrac,irho
+ use settings_data, only:ncolumns,ndimV,icoords,ndim,debugmode,ntypes,iverbose,fakedust
  use particle_data, only:dat,npartoftype,iamtype
  use geometry,      only:labelcoord
  use filenames,     only:ifileopen,nstepsinfile
@@ -174,6 +174,15 @@ subroutine adjust_data_codeunits
        endif
     endif
  endif
+ !
+ !--fake a set of dust particles from the one-fluid dust formulation
+ !
+ if (idustfrac.gt.0 .and. irho.gt.0 .and.            &
+          .not.(lenvironment('SPLASH_BARYCENTRIC')   &
+          .or. lenvironment('NSPLASH_BARYCENTRIC'))) then
+    fakedust = .true.
+    call fake_twofluids(1,nstepsinfile(ifileopen),ndim,ndimV,dat,npartoftype,iamtype)
+ endif
 
 end subroutine adjust_data_codeunits
 
@@ -264,7 +273,134 @@ pure subroutine shift_velocities(dat,np,ndimV,ncol,v0)
        dat(1:np,ivx+icol-1) = dat(1:np,ivx+icol-1) - v0(icol)
     enddo
  endif
- 
+
 end subroutine shift_velocities
 
+!------------------------------------------------------
+!
+! routine to fake a set of dust particles from
+! the one fluid dust method
+!
+!------------------------------------------------------
+subroutine fake_twofluids(istart,iend,ndim,ndimV,dat,npartoftype,iamtype)
+ use params,         only:int1
+ use labels,         only:idustfrac,idustfracsum,idustfrac_plot,ideltav_plot, &
+                          irho,ix,ih,ipmass,ivx,ideltav,ideltavsum
+ use mem_allocation, only:alloc
+ use particle_data,  only:maxpart,maxstep,maxcol
+ use settings_data,  only:iverbose,required
+ use settings_data,  only:ndusttypes
+ integer,                       intent(in)    :: istart,iend,ndim,ndimV
+ real,    dimension(:,:,:),     intent(inout), allocatable :: dat
+ integer, dimension(:,:),       intent(inout), allocatable :: npartoftype
+ integer(int1), dimension(:,:), intent(inout), allocatable :: iamtype
+ integer :: ndust,jdust,ntoti,i,j
+ integer :: idustfrac_temp,ideltav_temp
+ real    :: rhodust,rhogas,rhotot,dustfraci,gasfraci,pmassgas,pmassdust,pmassj
+ real, dimension(ndimV) :: veli,vgas,vdust,deltav,deltavsum
+ logical :: use_vels
+
+ if (idustfrac.gt.0 .and. irho.gt.0) then
+    !
+    !--determine which dust fraction is being used to create the fake dust particles
+    !
+    !if (iverbose >= 0) print*,' got dustfrac in column ',idustfrac
+    if (ndusttypes>1) then
+       if (idustfrac_plot == 0 ) then
+          idustfrac_temp = idustfracsum
+          idustfrac_plot = idustfracsum
+          ideltav_temp   = ideltavsum
+          ideltav_plot   = ideltavsum
+       else
+          idustfrac_temp = idustfrac_plot
+          ideltav_temp   = ideltav_plot
+       endif
+    else
+       idustfrac_temp = idustfrac
+       ideltav_temp   = ideltav
+    endif
+    !
+    !--create explicit dust phase from gas and one-fluid dust properties
+    !
+    do i=istart,iend
+       ntoti = sum(npartoftype(:,i))
+       if (.not.allocated(dat) .or. (ntoti + npartoftype(1,i)).gt.maxpart) then
+          call alloc(ntoti + npartoftype(1,i),maxstep,maxcol,mixedtypes=.true.)
+       endif
+       if (npartoftype(2,i) > 0) cycle
+       ndust = 0
+       !--zero the properties of newly created dust particles
+       dat(ntoti+1:ntoti+npartoftype(1,i),:,i) = 0.
+       if (idustfrac_temp > size(dat(1,:,1))) then
+          print*,' ERROR: idustfrac out of range: cannot create fake dust particles'
+          return
+       endif
+       use_vels = (ideltav_temp.gt.0 .and. ivx.gt.0 .and. ndimV.gt.0)
+       do j=1,ntoti
+          if (iamtype(j,i).eq.1) then
+             ndust = ndust + 1 ! one dust particle for every gas particle
+             rhotot  = dat(j,irho,i)
+             dustfraci = dat(j,idustfrac_temp,i)
+             if (idustfracsum == 0) then
+                gasfraci = 1. - dustfraci
+             else
+                gasfraci = 1. - dat(j,idustfracsum,i)
+             endif
+             rhogas  = rhotot*gasfraci
+             rhodust = rhotot*dustfraci
+             !--replace global properties with gas-only stuff
+             dat(j,irho,i) = rhogas
+             !--copy x, smoothing length onto dust particle
+             jdust = ntoti + ndust
+
+             !--fill in dust properties
+             if (ndim.gt.0) dat(jdust,ix(1:ndim),i) = dat(j,ix(1:ndim),i)
+             if (ih.gt.0)   dat(jdust,ih,i)         = dat(j,ih,i)
+             if (irho.gt.0) dat(jdust,irho,i)       = rhodust
+             if (idustfracsum == 0) then
+                dat(jdust,idustfrac,i) = dustfraci
+             else
+                dat(jdust,idustfracsum:idustfracsum+ndusttypes,i) = &
+                    dat(j,idustfracsum:idustfracsum+ndusttypes,i)
+             endif
+             iamtype(ntoti + ndust,i) = 2
+
+             !--particle masses
+             if (ipmass.gt.0) then
+                pmassj    = dat(j,ipmass,i)
+                pmassgas  = pmassj*gasfraci
+                pmassdust = pmassj*dustfraci
+                dat(j,ipmass,i)     = pmassgas
+                dat(jdust,ipmass,i) = pmassdust
+             endif
+
+             !--velocities
+             if (use_vels) then
+                veli(:)      = dat(j,ivx:ivx+ndimV-1,i)
+                deltav(:)    = dat(j,ideltav_temp:ideltav_temp+ndimV-1,i)
+                if (ndusttypes>1 .and. idustfrac_temp/=idustfracsum) then
+                   deltavsum(:) = dat(j,ideltavsum:ideltavsum+ndimV-1,i)
+                   vgas(:)      = veli(:) - (1. - gasfraci)*deltavsum(:)
+                   vdust(:)     = veli(:) + deltav(:) - (1. - gasfraci)*deltavsum(:)
+                else
+                   vgas(:)      = veli(:) - (1. - gasfraci)*deltav(:)
+                   vdust(:)     = veli(:) + gasfraci*deltav(:)
+                endif
+                dat(j,ivx:ivx+ndimV-1,i)     = vgas(:)
+                dat(jdust,ivx:ivx+ndimV-1,i) = vdust(:)
+             endif
+          endif
+       enddo
+       if (iverbose.ge.1) then
+          print "(a,i8,a)",' Creating ',ndust,' fake dust particles (set SPLASH_BARYCENTRIC=yes to plot barycentric values)'
+          if (.not.use_vels .and. ivx > 0) print "(a)",' WARNING: deltav not found in one fluid dust data: cannot get vels'
+       endif
+       npartoftype(2,i) = npartoftype(2,i) + ndust
+    enddo
+ else
+    print "(a)",' ERROR: could not locate dust-to-gas ratio and/or density'
+ endif
+
+end subroutine fake_twofluids
+
 end module adjustdata
