@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2015 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2016 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -39,6 +39,7 @@ module exact
   !--options used to plot the exact solution line
   !
   integer :: maxexactpts, iExactLineColour, iExactLineStyle,iPlotExactOnlyOnPanel
+  integer :: iNormaliseErrors
   logical :: iApplyTransExactFile,iCalculateExactErrors,iPlotResiduals
   logical :: iApplyUnitsExactFile
   real :: fracinsetResiduals,residualmax
@@ -89,6 +90,8 @@ module exact
   integer, parameter :: iexact_rochelobe = 15
   !--C-shock
   real :: machs,macha
+  !--planet-disc interaction
+  real :: HonR,rplanet
   !--gamma, for manual setting
   real :: gamma_exact
   logical :: use_gamma_exact
@@ -98,7 +101,7 @@ module exact
   namelist /exactopts/ iexactplotx,iexactploty,filename_exact,maxexactpts, &
        iExactLineColour,iExactLineStyle,iApplyTransExactFile,iCalculateExactErrors, &
        iPlotResiduals,fracinsetResiduals,residualmax,iPlotExactOnlyOnPanel,&
-       iApplyUnitsExactFile
+       iApplyUnitsExactFile,iNormaliseErrors
 
   namelist /exactparams/ ampl,lambda,period,iwaveploty,iwaveplotx,xzero, &
        htstar,atstar,ctstar,alphatstar,betatstar,ctstar1,ctstar2, &
@@ -107,7 +110,8 @@ module exact
        iprofile,Msphere,rsoft,icolpoten,icolfgrav,Mstar,Rtorus,distortion, &
        Mring,Rring,viscnu,nfunc,funcstring,cs,Kdrag,rhozero,rdust_to_gas, &
        mprim,msec,ixcolfile,iycolfile,xshock,totmass,machs,macha,&
-       use_sink_data,xprim,xsec,nfiles,gamma_exact,use_gamma_exact
+       use_sink_data,xprim,xsec,nfiles,gamma_exact,use_gamma_exact,&
+       HonR,rplanet
 
   public :: defaults_set_exact,submenu_exact,options_exact,read_exactparams
   public :: exact_solution
@@ -187,6 +191,10 @@ contains
 !   C-shock
     machs = 50. ! sonic Mach number
     macha = 5.  ! Alfvenic Mach number
+!   planet-disc interaction
+    HonR = 0.05
+    rplanet = 1.
+    
 !   gamma, if not read from file
     gamma_exact = 5./3.
     use_gamma_exact = .false.
@@ -201,6 +209,7 @@ contains
     iApplyTransExactFile = .true. ! false if exact from file is already logged
     iApplyUnitsExactFile = .false.
     iCalculateExactErrors = .true.
+    iNormaliseErrors = 1
     iPlotResiduals = .false.
     fracinsetResiduals = 0.15
     residualmax = 0.0
@@ -242,8 +251,9 @@ contains
            '13) special relativistic shock tube', /, &
            '14) dusty waves', /, &
            '15) Roche lobes/potential ',/, &
-           '16) C-shock ')
-    call prompt('enter exact solution to plot',iexact,0,16)
+           '16) C-shock ',/, &
+           '17) Planet-disc interaction')
+    call prompt('enter exact solution to plot',iexact,0,17)
     print "(a,i2)",'plotting exact solution number ',iexact
     !
     !--enter parameters for various exact solutions
@@ -363,6 +373,7 @@ contains
        !
        call read_exactparams(iexact,trim(rootname(1)),ierr)
        if (ierr.ne.0) then
+          call prompt('enter initial x position of shock',xshock)
           call prompt('enter density to left of shock   ',rho_L,0.0)
           call prompt('enter density to right of shock  ',rho_R,0.0)
           call prompt('enter pressure to left of shock  ',pr_L,0.0)
@@ -486,6 +497,9 @@ contains
     case(16)
        call prompt('enter sonic Mach number',machs,0.)
        call prompt('enter Alfvenic Mach number ',macha,0.)
+    case(17)
+       call prompt('enter disc aspect ratio (H/R)',HonR,0.,1.)
+       !call prompt('enter planet orbital radius ',rplanet,0.)
     end select
     
     if (prompt_for_gamma) then
@@ -509,7 +523,12 @@ contains
     call prompt('enter number of exact solution points ',maxexactpts,10,1000000)
     call prompt('enter line colour ',iExactLineColour,1,plotlib_maxlinecolour)
     call prompt('enter line style  ',iExactLineStyle,1,plotlib_maxlinestyle)
-    call prompt('calculate error norms? ',iCalculateExactErrors)
+    call prompt('calculate error norms?',iCalculateExactErrors)
+    if (iCalculateExactErrors) then
+       print "(/,' 0 : not normalised  L1 = 1/N \sum |y - y_exact|',/,"// &
+                "' 1 : normalised      L1 = 1/N 1/max(y_exact) \sum |y - y_exact|')"
+       call prompt('enter choice of error norm',iNormaliseErrors)
+    endif
     if (iCalculateExactErrors) then
        call prompt('plot residuals (as inset in main plot)?',iPlotResiduals)
        if (iPlotResiduals) then
@@ -551,7 +570,7 @@ contains
     character(len=30)  :: var
     logical            :: iexist
 
-    idash = index(rootname,'_')
+    idash = index(rootname,'_',back=.true.)
     if (idash.eq.0) idash = len_trim(rootname)+1
 
     select case(iexact)
@@ -639,6 +658,8 @@ contains
                    read(line(ieq+1:),*,iostat=ierrs(1)) v_L
                 case('vxright')
                    read(line(ieq+1:),*,iostat=ierrs(1)) v_R
+                case('dtg')
+                   read(line(ieq+1:),*,iostat=ierrs(1)) rdust_to_gas
                 end select
              endif
           enddo
@@ -718,10 +739,15 @@ contains
        return
     case(14)
        !
-       !--dustywave parameters from ndspmhd input file
+       !--dustywave parameters from ndspmhd or Phantom input file
        !
        filename = trim(rootname(1:idash-1))//'.in'
-       linenum = get_line_containing(filename,'Kdrag')
+       var = 'Kdrag' !ndspmhd
+       linenum = get_line_containing(filename,trim(var))
+       if (linenum==0) then
+          var = 'K_code' !phantom
+          linenum = get_line_containing(filename,trim(var))
+       endif
        open(unit=19,file=filename,status='old',iostat=ierr)
        if (ierr.eq.0) then
           do i=1,linenum-1
@@ -729,13 +755,17 @@ contains
           enddo
           read(19,"(a)",iostat=ierr) line
           if (ierr.eq.0) then
-             k = index(line,'Kdrag =')
-             if (k > 0) then
-                read(line(k+7:),*,iostat=ierr) Kdrag
-                print*,'>> read Kdrag = ',Kdrag,' from '//trim(filename)
-             elseif (ierr.eq.0) then
+             k = index(line,trim(var))
+             if (k > 0) read(line(k+len_trim(var)+2:),*,iostat=ierr) Kdrag
+             if (ierr.eq.0) then
+                print*,'>> read '//trim(var)//' = ',Kdrag,' from '//trim(filename)
+             else
                 read(line,*,iostat=ierr) idrag, idum, idum, Kdrag
-                print*,'>> read Kdrag = ',Kdrag,' from '//trim(filename)
+                if (ierr.eq.0) then
+                   print*,'>> read '//trim(var)//' = ',Kdrag,' from old-style '//trim(filename)
+                else
+                   print*,'>> error reading Kdrag from '//trim(filename)
+                endif
              endif
           else
              print*,'>> error reading Kdrag from '//trim(filename)
@@ -804,6 +834,7 @@ contains
     use rochelobe,       only:exact_rochelobe
     use gresho,          only:exact_gresho
     use Cshock,          only:exact_Cshock
+    use planetdisc,      only:exact_planetdisc
     use transforms,      only:transform,transform_inverse
     use plotlib,         only:plot_qci,plot_qls,plot_sci,plot_sls,plot_line,plotlib_maxlinestyle
     integer, intent(in) :: iexact,iplotx,iploty,itransx,itransy,igeom
@@ -822,6 +853,7 @@ contains
     real, allocatable :: xexact(:),yexact(:),xtemp(:)
     real :: dx,timei,gammai
     character(len=len(filename_exact)) :: filename_tmp
+    logical :: iplot_type_tmp(maxparttypes)
 
     !
     !--change line style and colour settings, but save old ones
@@ -928,22 +960,22 @@ contains
     case(3)! shock tube
        if (iplotx.eq.ix(1) .and. igeom.le.1) then
           if (iploty.eq.irho) then
-             call exact_shock(1,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+             call exact_shock(1,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.ipr) then
-             call exact_shock(2,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+             call exact_shock(2,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)   
           elseif (iploty.eq.ivx) then
-              call exact_shock(3,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(3,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.iutherm) then
-              call exact_shock(4,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(4,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.ideltav) then
-              call exact_shock(5,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(5,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           elseif (iploty.eq.idustfrac) then
-              call exact_shock(6,timei,gammai,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
+              call exact_shock(6,timei,gammai,xshock,rho_L,rho_R,pr_L,pr_R,v_L,v_R, &
                               rdust_to_gas,xexact,yexact,ierr)
           endif
        endif
@@ -1183,24 +1215,24 @@ contains
              !--plot gas solution and calculate errors
              call exact_dustywave(1,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
              call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
-                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1)
+                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1,matchtype=1)
              !--plot dust solution
              if (Kdrag > 0.) then
                 call exact_dustywave(2,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
                 call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot, &
-                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,err=.false.)
+                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,matchtype=2)
              endif
              ierr = 1
           elseif (iploty.eq.irho) then
              !--plot gas solution and calculate errors
              call exact_dustywave(3,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
              call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,&
-                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1)
+                                      itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=1,matchtype=1)
              !--plot dust solution
              if (Kdrag > 0.) then
                 call exact_dustywave(4,timei,ampl,cs,Kdrag,lambda,xzero,rhozero,rhozero*rdust_to_gas,xexact,yexact,ierr)
                 call plot_exact_solution(itransx,itransy,iexactpts,npart,xexact,yexact,xplot,yplot,&
-                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,err=.false.)
+                                         itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls=2,matchtype=2)
              endif
              ierr = 1
           endif
@@ -1223,6 +1255,10 @@ contains
              call exact_Cshock(5,timei,gammai,machs,macha,xmin,xmax,xexact,yexact,ierr)
           endif
        endif
+    case(17) ! planet-disc interaction
+       if (ndim.ge.2 .and. iplotx.eq.ix(1) .and. iploty.eq.ix(2)) then
+          call exact_planetdisc(igeom,timei,HonR,rplanet,xexact,yexact,ierr)
+       endif
     end select
 
     !----------------------------------------------------------
@@ -1251,7 +1287,7 @@ contains
   ! and calculate errors with respect to the data
   !------------------------------------------------------------------
   subroutine plot_exact_solution(itransx,itransy,iexactpts,np,xexact,yexact,xplot,yplot,&
-                                 itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls,err)
+                                 itag,iamtype,noftype,iplot_type,xmin,xmax,imarker,iaxisy,ls,matchtype,err)
    use transforms, only:transform,transform_inverse
    use plotlib,    only:plot_line,plot_sls,plot_sci,plot_qci
    use params,     only:int1,maxparttypes
@@ -1262,11 +1298,11 @@ contains
    integer(int1), intent(in) :: iamtype(:)
    integer,       intent(in) :: noftype(maxparttypes)
    logical,       intent(in) :: iplot_type(maxparttypes)
-   integer,       intent(in), optional :: ls
+   integer,       intent(in), optional :: ls, matchtype
    logical,       intent(in), optional :: err
    real :: residuals(np),ypart(np)
    real :: errL1,errL2,errLinf
-   integer :: iused,ierr,iCurrentColour
+   integer :: iused,ierr,iCurrentColour,imatchtype
    logical :: plot_err
    character(len=12) :: str1,str2
 
@@ -1286,6 +1322,11 @@ contains
    else
       plot_err = .true.   
    endif
+   if (present(matchtype)) then
+      imatchtype = matchtype
+   else
+      imatchtype = 0
+   endif
 
    if (iCalculateExactErrors .and. plot_err) then
       !--untransform y axis again for error calculation
@@ -1296,7 +1337,7 @@ contains
       !--calculate errors
       call calculate_errors(xexact(1:iexactpts),yexact(1:iexactpts),xplot(1:np),ypart,&
                             itag,iamtype,noftype,iplot_type,xmin,xmax,residuals, &
-                            errL1,errL2,errLinf,iused)
+                            errL1,errL2,errLinf,iused,imatchtype)
       if (iused.ne.np) then
          write(str1,"(i12)",iostat=ierr) iused
          write(str2,"(i12)",iostat=ierr) np
@@ -1319,7 +1360,7 @@ contains
   ! Calculate various error norms 
   !--------------------------------
   subroutine calculate_errors(xexact,yexact,xpts,ypts,itag,iamtype,noftype,iplot_type,&
-                              xmin,xmax,residual,errL1,errL2,errLinf,iused)
+                              xmin,xmax,residual,errL1,errL2,errLinf,iused,imatchtype)
    use part_utils, only:igettype
    use params,     only:int1,maxparttypes
    real,    intent(in)  :: xexact(:),yexact(:),xpts(:),ypts(:),xmin,xmax
@@ -1330,6 +1371,7 @@ contains
    real,    intent(out) :: residual(size(xpts))
    real,    intent(out) :: errL1,errL2,errLinf
    integer, intent(out) :: iused
+   integer, intent(in)  :: imatchtype
    integer :: i,j,npart,nerr,nused,itype
    real    :: xi,dy,dx,yexacti,err1,ymax
    logical :: mixedtypes
@@ -1354,12 +1396,12 @@ contains
          else
             itype = igettype(i,noftype)
          endif
-         if (iplot_type(itype)) then
+         if (iplot_type(itype) .and. ((imatchtype==0).or.(itype==imatchtype))) then
             !
             !--find nearest point in exact solution table
             !
             do j=1,size(xexact)-1
-               if (xexact(j).lt.xi .and. xexact(j+1).gt.xi) then
+               if (xexact(j).le.xi .and. xexact(j+1).gt.xi) then
                   if (abs(residual(i)).gt.tiny(residual)) nerr = nerr + 1
                   !--linear interpolation from tabulated exact solution
                   dy = yexact(j+1) - yexact(j)
@@ -1367,7 +1409,7 @@ contains
                   if (dx.gt.0.) then
                      yexacti = yexact(j) + dy/dx*(xi - xexact(j))
                      residual(i) = ypts(i) - yexacti
-                  elseif (dy.gt.0.) then
+                  elseif (dy.ge.0.) then
                      yexacti = yexact(j)
                      residual(i) = ypts(i) - yexacti
                   else
@@ -1390,10 +1432,15 @@ contains
    !
    !--normalise errors (use maximum y value)
    !
-   if (ymax.gt.tiny(ymax) .and. nused > 0) then
-      errL1 = errL1/(nused*ymax)
-      errL2 = sqrt(errL2/(nused*ymax**2))
-      errLinf = errLinf/ymax
+   if (nused > 0) then
+      if (iNormaliseErrors==1 .and. ymax > tiny(ymax)) then
+         errL1 = errL1/(nused*ymax)
+         errL2 = sqrt(errL2/(nused*ymax**2))
+         errLinf = errLinf/ymax
+      else
+         errL1 = errL1/nused
+         errL2 = sqrt(errL2/(nused))
+      endif
    else
       print "(a)",' error normalising errors'
       errL1 = 0.
