@@ -45,6 +45,7 @@ module labels
  integer :: isurfdens,itoomre
  integer :: ipdf,icolpixmap
  integer :: irhorestframe,idustfrac,ideltav
+ integer :: idustfracsum,ideltavsum,idustfrac_plot,ideltav_plot
 
  public
 
@@ -85,6 +86,12 @@ subroutine reset_columnids
  iradenergy = 0
  icolpixmap = 0
  irhorestframe = 0
+ idustfrac = 0
+ idustfracsum = 0
+ idustfrac_plot = 0
+ ideltav = 0
+ ideltavsum = 0
+ ideltav_plot = 0
 
  return
 end subroutine reset_columnids
@@ -106,6 +113,44 @@ logical function is_coord(icol,ndim)
 
 end function is_coord
 
+!--------------------------------------------------------------
+!
+!  query function for whether column is a spatial coordinate
+!  returns the location of the third dimension in the
+!  list of columns
+!
+!--------------------------------------------------------------
+integer function get_z_dir(ndim,iplotx,iploty) result(iplotz)
+ implicit none
+ integer, intent(in) :: ndim,iplotx,iploty
+ integer :: i
+
+ iplotz = 0
+ do i=1,ndim
+    if ((iplotx.ne.iploty.and. &
+        (ix(i).ne.iplotx).and.(ix(i).ne.iploty))) iplotz = ix(i)
+ enddo
+
+end function get_z_dir
+
+!--------------------------------------------------------------
+!
+!  query function for which coordinate is z
+!  returns an integer between 1 and ndim
+!
+!--------------------------------------------------------------
+integer function get_z_coord(ndim,iplotx,iploty) result(iplotz)
+ implicit none
+ integer, intent(in) :: ndim,iplotx,iploty
+ integer :: i
+
+ i = get_z_dir(ndim,iplotx,iploty)
+ iplotz = i - ix(1) + 1
+ if (iplotz < 0)    iplotz = 1
+ if (iplotz > ndim) iplotz = ndim
+
+end function get_z_coord
+
 !-----------------------------------------------------------------
 !
 !  utility to strip spaces, escape sequences and
