@@ -69,7 +69,7 @@ module asciiread
 
 end module asciiread
 
-subroutine read_data(rootname,indexstart,nstepsread)
+subroutine read_data(rootname,indexstart,ipos,nstepsread)
   use particle_data,  only:dat,npartoftype,time,gamma,maxpart,maxcol,maxstep,iamtype
   use params
   use settings_data,  only:ndim,ndimV,ncolumns,ncalc,iverbose,ntypes
@@ -79,7 +79,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
   use asciiread,      only:icoltype
   use labels,         only:labeltype,print_types
   implicit none
-  integer, intent(in)          :: indexstart
+  integer, intent(in)          :: indexstart,ipos
   integer, intent(out)         :: nstepsread
   character(len=*), intent(in) :: rootname
   integer :: i,j,ierr,iunit,ncolstep,ncolenv,nerr,iheader_time,iheader_gamma
@@ -370,7 +370,8 @@ subroutine set_labels
            ndim = 3
            ix(3) = i
         endif
-        if (labeli(1:3).eq.'den' .or. index(labeli,'rho').ne.0 .or. labeli(1:3).eq.'\gr') then
+        if (labeli(1:3).eq.'den' .or. index(labeli,'rho').ne.0 .or. labeli(1:3).eq.'\gr' .or. &
+            (index(labeli,'density').ne.0 .and. irho==0)) then
            irho = i
         elseif (labeli(1:5).eq.'pmass' .or. labeli(1:13).eq.'particle mass' &
                 .or. index(labeli,'mass').ne.0) then
@@ -382,9 +383,10 @@ subroutine set_labels
                 .or. labeli(1:6).eq.'smooth')) then
            ih = i
         elseif (trim(labeli).eq.'u'.or.labeli(1:6).eq.'utherm' &
-            .or.trim(labeli).eq.'internal energy') then
+            .or.(index(labeli,'internal energy').ne.0 .and. iutherm==0)) then
            iutherm = i
-        elseif (labeli(1:2).eq.'pr' .or. trim(labeli).eq.'p') then
+        elseif (labeli(1:2).eq.'pr' .or. trim(labeli).eq.'p' .or. &
+               (index(labeli,'pressure').ne.0 .and. ipr==0)) then
            ipr = i
         elseif (ivx.eq.0 .and. labeli(1:1).eq.'v') then
            ivx = i
