@@ -47,7 +47,7 @@
 ! in the module 'particle_data'
 !-------------------------------------------------------------------------
 
-subroutine read_data(rootname,indexstart,nstepsread)
+subroutine read_data(rootname,indexstart,ipos,nstepsread)
   use exact,          only:hfact
   use particle_data,  only:npartoftype,time,gamma,dat,maxpart,maxstep,maxcol,iamtype
   use params
@@ -59,7 +59,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
   use system_utils,   only:lenvironment
   use labels,         only:labeltype,print_types
   implicit none
-  integer,          intent(in)  :: indexstart
+  integer,          intent(in)  :: indexstart,ipos
   integer,          intent(out) :: nstepsread
   character(len=*), intent(in)  :: rootname
   character(len=len(rootname)+4) :: datfile
@@ -317,10 +317,10 @@ ndimV = ndimV_max
 
 call set_labels
 
-if (iformat.eq.5 .and. .not.lenvironment('NSPLASH_BARYCENTRIC')) then
-   call fake_twofluids
-   iformat = 1
-endif
+!if (iformat.eq.5 .and. .not.lenvironment('NSPLASH_BARYCENTRIC')) then
+!   call fake_twofluids
+!   iformat = 1
+!endif
 
 if (any(npartoftype(2:,ilast) > 0)) call print_types(npartoftype(:,ilast),labeltype)
 
@@ -332,74 +332,6 @@ return
 print*,' *** data file empty : no timesteps ***'
 return
 
-contains
-
-subroutine fake_twofluids
- use labels, only:idustfrac,irho,ix,ih,ipmass,ivx,ideltav
- implicit none
- integer :: ndust,jdust
- real    :: rhodust,rhogas,rhotot,dustfraci,pmassgas,pmassdust,pmassj
- real, dimension(ndimV) :: veli,vgas,vdust,deltav
-
- if (idustfrac.gt.0 .and. irho.gt.0) then
-    do i=indexstart,indexstart+nstepsread-1
-       ntoti = sum(npartoftype(:,i))
-       if (.not.allocated(dat) .or. (ntoti + npartoftype(1,i)).gt.maxpart) then
-          call alloc(ntoti + npartoftype(1,i),maxstep,maxcol,mixedtypes=.true.)
-       endif
-       ndust = 0
-       !--zero the properties of newly created dust particles
-       dat(ntoti+1:ntoti+npartoftype(1,i),:,i) = 0.
-       do j=1,ntoti
-          if (iamtype(j,i).eq.1) then
-             ndust = ndust + 1 ! one dust particle for every gas particle
-             rhotot  = dat(j,irho,i)
-             dustfraci = dat(j,idustfrac,i)
-             rhogas  = rhotot*(1. - dustfraci)
-             rhodust = rhotot*dustfraci
-             !--replace global properties with gas-only stuff
-             dat(j,irho,i) = rhogas
-             !--copy x, smoothing length onto dust particle
-             jdust = ntoti + ndust
-             
-             !--fill in dust properties
-             if (ndim.gt.0) dat(jdust,ix(1:ndim),i) = dat(j,ix(1:ndim),i)
-             if (ih.gt.0)   dat(jdust,ih,i)         = dat(j,ih,i)
-             if (irho.gt.0) dat(jdust,irho,i)       = rhodust
-             iamtype(ntoti + ndust,i) = 2 
-
-             !--particle masses
-             if (ipmass.gt.0) then
-                pmassj    = dat(j,ipmass,i)
-                pmassgas  = pmassj*(1. - dustfraci)
-                pmassdust = pmassj*dustfraci
-                dat(j,ipmass,i)     = pmassgas
-                dat(jdust,ipmass,i) = pmassdust
-             endif
-
-             !--velocities
-             if (ideltav.gt.0 .and. ivx.gt.0 .and. ndimV.gt.0) then
-                veli(:)   = dat(j,ivx:ivx+ndimV-1,i)
-                deltav(:) = dat(j,ideltav:ideltav+ndimV-1,i)
-                vgas(:)   = veli(:) - rhodust/rhotot*deltav(:)
-                vdust(:)  = veli(:) + rhogas/rhotot*deltav(:)
-                dat(j,ivx:ivx+ndimV-1,i)     = vgas(:)
-                dat(jdust,ivx:ivx+ndimV-1,i) = vdust(:)
-             endif
-          endif
-       enddo
-       if (iverbose.ge.1) then
-          print "(a,i10,a)",' Creating ',ndust,' fictional dust particles...'
-          print "(a)",' (set NSPLASH_BARYCENTRIC=yes to plot barycentric values)'
-       endif
-       npartoftype(2,i) = npartoftype(2,i) + ndust
-    enddo
- else
-    print "(a)",' ERROR: could not locate dust-to-gas ratio and/or density'
- endif
-
-end subroutine fake_twofluids
-
 end subroutine read_data
 
 !!------------------------------------------------------------
@@ -412,7 +344,7 @@ subroutine set_labels
              irhorestframe,idustfrac,ideltav
  use params
  use settings_data, only:ndim,ndimV,iformat,ntypes, &
-                    UseTypeInRenderings
+                    UseTypeInRenderings,ncolumns
  use geometry, only:labelcoord
  implicit none
  integer :: i,icol
@@ -506,13 +438,15 @@ subroutine set_labels
     icol = icol + ndimV
     iBfirst = 0
 
-    iamvec(icol+1:icol+ndimV) = icol + 1
-    labelvec(icol+1:icol+ndimV) = 'del^{2} v'
-    icol = icol + ndimV
+    if (ncolumns > 20) then
+       iamvec(icol+1:icol+ndimV) = icol + 1
+       labelvec(icol+1:icol+ndimV) = 'del^{2} v'
+       icol = icol + ndimV
 
-    iamvec(icol+1:icol+ndimV) = icol + 1
-    labelvec(icol+1:icol+ndimV) = 'grad (div {\bf v})'
-    icol = icol + ndimV
+       iamvec(icol+1:icol+ndimV) = icol + 1
+       labelvec(icol+1:icol+ndimV) = 'grad (div {\bf v})'
+       icol = icol + ndimV
+    endif
  endif
  if (iformat.eq.5) then
     icol = icol + 1
