@@ -68,8 +68,8 @@ module sphNGread
  real :: tfreefall
  integer :: istartmhd,istartrt,nmhd,idivvcol,icurlvxcol,icurlvycol,icurlvzcol
  integer :: nhydroreal4,istart_extra_real4
- integer :: nhydroarrays,nmhdarrays
- logical :: phantomdump,smalldump,mhddump,rtdump,usingvecp,igotmass,h2chem,rt_in_header
+ integer :: nhydroarrays,nmhdarrays,ndustarrays
+ logical :: phantomdump,smalldump,mhddump,rtdump,usingvecp,igotmass,h2chem,rt_in_header,onefluid_dust
  logical :: usingeulr,cleaning
  logical :: batcode,tagged,debug
  integer, parameter :: maxarrsizes = 10
@@ -305,10 +305,10 @@ contains
  ! Extract various options from the fileident string
  !----------------------------------------------------------------------
  subroutine get_options_from_fileident(fileident,smalldump,tagged,phantomdump,&
-                       usingvecp,usingeulr,cleaning,h2chem,rt_in_header,batcode)
+                       usingvecp,usingeulr,cleaning,h2chem,use_dustfrac,rt_in_header,batcode)
   character(len=*), intent(in) :: fileident
   logical,          intent(out) :: smalldump,tagged,phantomdump,batcode
-  logical,          intent(out) :: usingvecp,usingeulr,cleaning,h2chem,rt_in_header
+  logical,          intent(out) :: usingvecp,usingeulr,cleaning,h2chem,use_dustfrac,rt_in_header
 
   smalldump = .false.
   phantomdump = .false.
@@ -319,6 +319,7 @@ contains
   rt_in_header = .false.
   batcode = .false.
   tagged = .false.
+  use_dustfrac = .false.
   if (fileident(1:1).eq.'S') then
      smalldump = .true.
   endif
@@ -345,12 +346,78 @@ contains
   if (index(fileident,'RT=on').ne.0) then
      rt_in_header = .true.
   endif
+  if (index(fileident,'dust').ne.0) then
+     use_dustfrac = .true.
+  endif
   if (index(fileident,'This is a test').ne.0) then
      batcode = .true.
   endif
 
  end subroutine get_options_from_fileident
 
+ !----------------------------------------------------------------------
+ ! Set position of header items manually for older (untagged) formats
+ !----------------------------------------------------------------------
+ subroutine fake_header_tags(nreals,realarr,tagsreal)
+  integer, intent(in) :: nreals
+  real,    intent(in) :: realarr(nreals)
+  character(len=*), intent(out) :: tagsreal(:)
+  integer, parameter :: ilocbinary = 24
+  integer :: ipos
+  !
+  !--set the tags manually for older formats
+  !  (but only the ones we care about)
+  !
+  if (phantomdump) then
+     tagsreal(1) = 'time'
+     tagsreal(3) = 'gamma'
+     tagsreal(4) = 'rhozero'
+     tagsreal(6) = 'hfact'
+     tagsreal(7) = 'tolh'
+     tagsreal(15:19) = 'massoftype'
+     if (nreals.ge.ilocbinary + 14) then
+        if (nreals.ge.ilocbinary + 15) then
+           ipos = ilocbinary
+        else
+           print*,'*** WARNING: obsolete header format for external binary information ***'
+           ipos = ilocbinary + 1
+        endif
+        if (debug) print*,'DEBUG: reading binary information from header ',ilocbinary
+        if (any(realarr(ilocbinary:ilocbinary+14).ne.0.)) then
+           tagsreal(ipos:ipos+2) = (/'x1','y1','z1'/)
+           if (nreals.ge.ilocbinary+15) then
+              tagsreal(ipos+3:ipos+9) = (/'m1','h1','x2','y2','z2','m2','h2'/)
+              ipos = ipos + 10
+           else
+              tagsreal(ipos+3:ipos+7) = (/'h1','x2','y2','z2','h2'/)
+              ipos = ipos + 8
+           endif
+           tagsreal(ipos:ipos+5) = (/'vx1','vy1','vz1','vx2','vy2','vz2'/)
+        endif
+     endif
+  elseif (batcode) then
+     tagsreal(1) = 'time'
+     tagsreal(3) = 'gamma'
+     tagsreal(4) = 'radL1'
+     tagsreal(5) = 'PhiL1'
+     tagsreal(15) = 'Er'
+  else
+     tagsreal(1) = 'gt'
+     tagsreal(2) = 'dtmax'
+     tagsreal(3) = 'gamma'
+     tagsreal(4) = 'rhozero'
+     tagsreal(5) = 'RK2'
+     if (smalldump) then ! sphNG small dump
+        if (nreals.eq.15) then
+           tagsreal(15) = 'pmassinitial'
+        else
+           tagsreal(23) = 'pmassinitial'
+        endif
+     endif
+  endif
+
+ end subroutine fake_header_tags
+
  !----------------------------------------------------------------------
  ! Routine to read the header of sphNG dump files and extract relevant
  ! information
@@ -358,6 +425,7 @@ contains
  subroutine read_header(iunit,iverbose,debug,doubleprec,&
                         npart,npartoftypei,n1,ntypes,nblocks,&
                         narrsizes,realarr,tagsreal,nreals,ierr)
+  use settings_data,  only:ndusttypes
   integer, intent(in)  :: iunit,iverbose
   logical, intent(in)  :: debug,doubleprec
   integer, intent(out) :: npart,npartoftypei(:),n1,ntypes,nblocks,narrsizes,nreals,ierr
@@ -479,41 +547,7 @@ contains
         print*,'WARNING: number of reals in header exceeds splash array limit, ignoring some'
         nreals = maxinblock
      endif
-     if (tagged) then
-        read(iunit,iostat=ierr) tagsreal(1:nreals)
-     else
-     !
-     !--set the tags manually for older formats
-     !  (but only the ones we care about)
-     !
-        if (phantomdump) then
-           tagsreal(1) = 'time'
-           tagsreal(3) = 'gamma'
-           tagsreal(4) = 'rhozero'
-           tagsreal(6) = 'hfact'
-           tagsreal(7) = 'tolh'
-           tagsreal(15:19) = 'massoftype'
-        elseif (batcode) then
-           tagsreal(1) = 'time'
-           tagsreal(3) = 'gamma'
-           tagsreal(4) = 'radL1'
-           tagsreal(5) = 'PhiL1'
-           tagsreal(15) = 'Er'
-        else
-           tagsreal(1) = 'gt'
-           tagsreal(2) = 'dtmax'
-           tagsreal(3) = 'gamma'
-           tagsreal(4) = 'rhozero'
-           tagsreal(5) = 'RK2'
-           if (smalldump) then ! sphNG small dump
-              if (nreals.eq.15) then
-                 tagsreal(15) = 'pmassinitial'
-              else
-                 tagsreal(23) = 'pmassinitial'
-              endif
-           endif
-        endif
-     endif
+     if (tagged) read(iunit,iostat=ierr) tagsreal(1:nreals)
      if (doubleprec) then
         read(iunit,iostat=ierr) real8arr(1:nreals)
         realarr(1:nreals) = real(real8arr(1:nreals))
@@ -521,6 +555,7 @@ contains
         read(iunit,iostat=ierr) real4arr(1:nreals)
         realarr(1:nreals) = real(real4arr(1:nreals))
      endif
+     if (.not.tagged) call fake_header_tags(nreals,realarr,tagsreal)
   endif
 !--real*4, real*8
   read(iunit,iostat=ierr) nreal4s
@@ -555,6 +590,8 @@ contains
      if (any(ierrs /= 0)) then
         print "(a)",' *** error reading units'
      endif
+     ! extract the number of dust arrays are in the file
+     if (onefluid_dust) ndusttypes = extract_ndusttypes(tags,tagsreal,intarr,nints)
   else
      if (nreal8s.ge.4) then
         read(iunit,iostat=ierr) udist,umass,utime,umagfd
@@ -685,56 +722,37 @@ contains
      !--if Phantom calculation uses the binary potential
      !  then read this as two point mass particles
      !
-     if (nreals.ge.ilocbinary + 14) then
-        if (nreals.ge.ilocbinary + 15) then
-           ipos = ilocbinary
-        else
-           print*,'*** WARNING: obsolete header format for external binary information ***'
-           ipos = ilocbinary + 1
+     if (any(tags(1:nreals)=='x1')) then
+        gotbinary = .true.
+        npartoftype(itypemap_sink_phantom) = npartoftype(itypemap_sink_phantom) + 2
+        ntotal = ntotal + 2
+        call extract('x1',dat(npart+1,ix(1)),realarr,tags,nreals,ierrs(1))
+        call extract('y1',dat(npart+1,ix(2)),realarr,tags,nreals,ierrs(2))
+        call extract('z1',dat(npart+1,ix(3)),realarr,tags,nreals,ierrs(3))
+        if (ipmass.gt.0) call extract('m1',dat(npart+1,ipmass),realarr,tags,nreals,ierrs(4))
+        call extract('h1',dat(npart+1,ih),realarr,tags,nreals,ierrs(4))
+        if (debug) print *,npart+1,npart+2
+        if (iverbose.ge.1) print *,'binary position:   primary: ',dat(npart+1,ix(1):ix(3))
+
+        call extract('x2',dat(npart+2,ix(1)),realarr,tags,nreals,ierrs(1))
+        call extract('y2',dat(npart+2,ix(2)),realarr,tags,nreals,ierrs(2))
+        call extract('z2',dat(npart+2,ix(3)),realarr,tags,nreals,ierrs(3))
+        if (ipmass.gt.0) call extract('m2',dat(npart+2,ipmass),realarr,tags,nreals,ierrs(4))
+        call extract('h2',dat(npart+2,ih),realarr,tags,nreals,ierrs(5))
+        if (iverbose.ge.1 .and. ipmass > 0 .and. ih > 0) then
+           print *,'                 secondary: ',dat(npart+2,ix(1):ix(3))
+           print *,' m1: ',dat(npart+1,ipmass),' m2:',dat(npart+2,ipmass),&
+                   ' h1: ',dat(npart+2,ipmass),' h2:',dat(npart+2,ih)
         endif
-        if (debug) print*,'DEBUG: reading binary information from header ',ilocbinary
-        if (any(realarr(ilocbinary:ilocbinary+14).ne.0.)) then
-           gotbinary = .true.
-           npartoftype(itypemap_sink_phantom) = npartoftype(itypemap_sink_phantom) + 2
-           ntotal = ntotal + 2
-           dat(npart+1,ix(1)) = realarr(ipos)
-           dat(npart+1,ix(2)) = realarr(ipos+1)
-           dat(npart+1,ix(3)) = realarr(ipos+2)
-           if (debug) print *,npart+1,npart+2
-           if (iverbose.ge.1) print *,'binary position:   primary: ',realarr(ipos:ipos+2)
-           if (nreals.ge.ilocbinary+15) then
-              if (ipmass.gt.0) dat(npart+1,ipmass) = realarr(ipos+3)
-              dat(npart+1,ih)     = realarr(ipos+4)
-              dat(npart+2,ix(1))  = realarr(ipos+5)
-              dat(npart+2,ix(2))  = realarr(ipos+6)
-              dat(npart+2,ix(3))  = realarr(ipos+7)
-              if (ipmass.gt.0) dat(npart+2,ipmass) = realarr(ipos+8)
-              dat(npart+2,ih)     = realarr(ipos+9)
-              if (iverbose.ge.1) then
-                 print *,'                 secondary: ',realarr(ipos+5:ipos+7)
-                 print *,' m1: ',realarr(ipos+3),' m2:',realarr(ipos+8),&
-                         ' h1: ',realarr(ipos+4),' h2:',realarr(ipos+9)
-              endif
-              ipos = ipos + 10
-           else
-              dat(npart+1,ih)    = realarr(ipos+3)
-              dat(npart+2,ix(1)) = realarr(ipos+4)
-              dat(npart+2,ix(2)) = realarr(ipos+5)
-              dat(npart+2,ix(3)) = realarr(ipos+6)
-              dat(npart+2,ih)    = realarr(ipos+7)
-              print *,'                 secondary: ', realarr(ipos+4:ipos+6)
-              ipos = ipos + 8
-           endif
-           if (ivx.gt.0) then
-              dat(npart+1,ivx)   = realarr(ipos)
-              dat(npart+1,ivx+1) = realarr(ipos+1)
-              dat(npart+1,ivx+2) = realarr(ipos+2)
-              dat(npart+2,ivx)   = realarr(ipos+3)
-              dat(npart+2,ivx+1) = realarr(ipos+4)
-              dat(npart+2,ivx+2) = realarr(ipos+5)
-           endif
-           npart  = npart  + 2
+        if (ivx.gt.0) then
+           call extract('vx1',dat(npart+1,ivx),realarr,tags,nreals,ierrs(1))
+           call extract('vy1',dat(npart+1,ivx+1),realarr,tags,nreals,ierrs(2))
+           call extract('vz1',dat(npart+1,ivx+2),realarr,tags,nreals,ierrs(3))
+           call extract('vx2',dat(npart+2,ivx),realarr,tags,nreals,ierrs(4))
+           call extract('vy2',dat(npart+2,ivx+1),realarr,tags,nreals,ierrs(5))
+           call extract('vz2',dat(npart+2,ivx+2),realarr,tags,nreals,ierrs(6))
         endif
+        npart  = npart  + 2
      endif
   else
      npartoftype(:) = 0
@@ -924,11 +942,12 @@ contains
   endif
  end subroutine guess_labels
 
- integer function assign_column(tag,iarr,ipos,ikind,imaxcolumnread) result(icolumn)
+ integer function assign_column(tag,iarr,ipos,ikind,imaxcolumnread,idustarr) result(icolumn)
   use labels, only:ih,irho,ix,ipmass
   character(len=lentag), intent(in) :: tag
   integer,               intent(in) :: iarr,ipos,ikind
   integer,               intent(inout) :: imaxcolumnread
+  integer,               intent(inout) :: idustarr
 
   if (tagged .and. len_trim(tag) > 0) then
      !
@@ -949,14 +968,24 @@ contains
         icolumn = ih
      case('rho')
         icolumn = irho
+     case('dustfracsum')
+        idustarr = idustarr + 1
+        icolumn = nhydroarrays + idustarr
+     case('dustfrac')
+        if (ndustarrays > 0) then
+           idustarr = idustarr + 1
+           icolumn = nhydroarrays + idustarr
+        else
+           icolumn = max(nhydroarrays + ndustarrays + nmhdarrays + 1,imaxcolumnread + 1)     
+        endif
      case('Bx')
-        icolumn = nhydroarrays + 1
+        icolumn = nhydroarrays + ndustarrays + 1
      case('By')
-        icolumn = nhydroarrays + 2
+        icolumn = nhydroarrays + ndustarrays + 2
      case('Bz')
-        icolumn = nhydroarrays + 3
+        icolumn = nhydroarrays + ndustarrays + 3
      case default
-        icolumn = max(nhydroarrays + nmhdarrays + 1,imaxcolumnread + 1)
+        icolumn = max(nhydroarrays + ndustarrays + nmhdarrays + 1,imaxcolumnread + 1)
         if (iarr==1) then
            if (ikind==4) then  ! real*4 array
               istart_extra_real4 = min(istart_extra_real4,icolumn)
@@ -1019,29 +1048,61 @@ contains
 
  end function assign_column
 
+!---------------------------------------------------------------
+! function to extract the number of dust arrays
+!---------------------------------------------------------------
+ integer function extract_ndusttypes(tags,tagsreal,intarr,nints) result(ndusttypes)
+  character(len=lentag), intent(in) :: tags(maxinblock),tagsreal(maxinblock)
+  integer, intent(in) :: intarr(:),nints
+  integer :: i,idust,ierr
+  logical :: igotndusttypes = .false.
+
+  ! Look for ndusttypes in the header
+  do i = 1,maxinblock
+     if (tags(i)=='ndusttypes') then
+         igotndusttypes = .true.
+     endif
+  enddo
+
+  ! Retreive/guess the value of ndusttypes
+  if (igotndusttypes) then
+     call extract('ndusttypes',idust,intarr,tags,nints,ierr)
+  else
+     ! For older files where ndusttypes is not output to the header
+     idust = 0
+     do i = 1,maxinblock
+        if (tagsreal(i)=='grainsize') idust = idust + 1
+     enddo
+     write(*,"(a)")    ' Warning! Could not find ndusttypes in header'
+     write(*,"(a,I4)") '          ...counting grainsize arrays...ndusttypes =',idust
+  endif
+  ndusttypes = idust
+
+ end function extract_ndusttypes
+
 end module sphNGread
 
 !----------------------------------------------------------------------
 !  Main read_data routine for splash
 !----------------------------------------------------------------------
-subroutine read_data(rootname,indexstart,nstepsread)
+subroutine read_data(rootname,indexstart,iposn,nstepsread)
   use particle_data,  only:dat,gamma,time,iamtype,npartoftype,maxpart,maxstep,maxcol,masstype
   !use params,         only:int1,int8
   use settings_data,  only:ndim,ndimV,ncolumns,ncalc,required,ipartialread,&
-                      lowmemorymode,ntypes,iverbose
+                      lowmemorymode,ntypes,iverbose,ndusttypes
   use mem_allocation, only:alloc
   use system_utils,   only:lenvironment,renvironment
-  use labels,         only:ipmass,irho,ih,ix,ivx,labeltype,print_types
+  use labels,         only:ipmass,irho,ih,ix,ivx,labeltype,print_types,idustfrac
   use calcquantities, only:calc_quantities
   use sphNGread
   implicit none
-  integer, intent(in)  :: indexstart
+  integer, intent(in)  :: indexstart,iposn
   integer, intent(out) :: nstepsread
   character(len=*), intent(in) :: rootname
   integer :: i,j,k,ierr,iunit
   integer :: intg1,int2,int3,ilocvx,iversion
   integer :: i1,iarr,i2,iptmass1,iptmass2
-  integer :: npart_max,nstep_max,ncolstep,icolumn,nptmasstot
+  integer :: npart_max,nstep_max,ncolstep,icolumn,idustarr,nptmasstot
   integer :: narrsizes
   integer :: nskip,ntotal,npart,n1,ngas,nreals
   integer :: iblock,nblocks,ntotblock,ncolcopy
@@ -1185,7 +1246,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
    mhddump = .false.
    rtdump = .false.
    call get_options_from_fileident(fileident,smalldump,tagged,phantomdump,&
-                                   usingvecp,usingeulr,cleaning,h2chem,rt_in_header,batcode)
+                                   usingvecp,usingeulr,cleaning,h2chem,onefluid_dust,rt_in_header,batcode)
    if (tagged .and. iversion < 1) print "(a)",'ERROR: got tagged format but iversion is ',iversion
 !
 !--read variables from header
@@ -1305,6 +1366,15 @@ subroutine read_data(rootname,indexstart,nstepsread)
       elseif (phantomdump .and. (nreal(1).lt.3 .or. nreal4(1).lt.1)) then
          print "(a)",' ERROR: x,y,z or h missing in phantom read'
       endif
+      if (onefluid_dust) then
+         if (ndusttypes>1) then
+            ndustarrays = ndusttypes + 1      !--the extra column is for dustfracsum
+         else
+            ndustarrays = 1
+         endif
+      else
+         ndustarrays = 0
+      endif
       if (narrsizes.ge.4) then
          nmhdarrays = 3 ! Bx,By,Bz
          nmhd = nreal(4) + nreal4(4) + nreal8(4) - nmhdarrays ! how many "extra" mhd arrays
@@ -1398,6 +1468,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
 !
    imaxcolumnread = 0
    icolumn = 0
+   idustarr   = 0
    istartmhd = 0
    istartrt = 0
    i1 = i2 + 1
@@ -1631,7 +1702,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
          do i=1,nreal(iarr)
             tagtmp = ''
             if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
-            icolumn = assign_column(tagtmp,iarr,i,6,imaxcolumnread)
+            icolumn = assign_column(tagtmp,iarr,i,6,imaxcolumnread,idustarr)
             if (tagged) tagarr(icolumn) = tagtmp
             if (debug)  print*,' reading real ',icolumn,' tag = ',trim(tagtmp)
             if (required(icolumn)) then
@@ -1686,7 +1757,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
          do i=1,nreal4(iarr)
             tagtmp = ''
             if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
-            icolumn = assign_column(tagtmp,iarr,i,4,imaxcolumnread)
+            icolumn = assign_column(tagtmp,iarr,i,4,imaxcolumnread,idustarr)
             if (debug) print*,'reading real4 ',icolumn,' tag = ',trim(tagtmp)
             if (tagged) tagarr(icolumn) = tagtmp
 
@@ -1754,7 +1825,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
          do i=1,nreal8(iarr)
             tagtmp = ''
             if (tagged) read(iunit,end=33,iostat=ierr) tagtmp
-            icolumn = assign_column(tagtmp,iarr,i,8,imaxcolumnread)
+            icolumn = assign_column(tagtmp,iarr,i,8,imaxcolumnread,idustarr)
             if (debug) print*,'reading real8 ',icolumn,' tag = ',trim(tagtmp)
             if (tagged) tagarr(icolumn) = tagtmp
             if (required(icolumn)) then
@@ -2096,9 +2167,11 @@ end subroutine read_data
 
 subroutine set_labels
   use labels, only:label,unitslabel,labelzintegration,labeltype,labelvec,iamvec, &
-              ix,ipmass,irho,ih,iutherm,ivx,iBfirst,idivB,iJfirst,icv,iradenergy,idustfrac
+              ix,ipmass,irho,ih,iutherm,ivx,iBfirst,idivB,iJfirst,icv,iradenergy,&
+              idustfrac,ideltav,idustfracsum,ideltavsum
   use params
-  use settings_data,   only:ndim,ndimV,ntypes,ncolumns,UseTypeInRenderings,debugmode
+  use settings_data,   only:ndim,ndimV,ntypes,ncolumns,UseTypeInRenderings,debugmode,&
+                            ndusttypes
   use geometry,        only:labelcoord
   use settings_units,  only:units,unitzintegration
   use sphNGread
@@ -2106,9 +2179,10 @@ subroutine set_labels
   use system_commands, only:get_environment
   use system_utils,    only:lenvironment
   implicit none
-  integer :: i
+  integer :: i,j
   real(doub_prec)   :: uergg
   character(len=20) :: string
+  character(len=20) :: dustfrac_string,deltav_string
 
   if (ndim.le.0 .or. ndim.gt.3) then
      print*,'*** ERROR: ndim = ',ndim,' in set_labels ***'
@@ -2130,7 +2204,8 @@ subroutine set_labels
      ih = 4       !  smoothing length
   endif
   irho = ih + 1     !  density
-  if (smalldump .and. nhydroreal4.ge.3) iutherm = irho+1
+  iutherm = 0
+  idustfrac = 0
 
   !
   !--translate array tags into column labels, where necessary
@@ -2165,8 +2240,14 @@ subroutine set_labels
            iJfirst = i
         case('psi')
            label(i) = '\psi'
+        case('dustfracsum')
+           idustfracsum = i
         case('dustfrac')
            idustfrac = i
+        case('deltavsumx')
+           ideltavsum = i
+        case('deltavx')
+           ideltav = i
         case('alpha')
            label(i) = '\alpha'
         case('alphaB')
@@ -2208,6 +2289,7 @@ subroutine set_labels
         end select
      enddo
   else
+     if (smalldump .and. nhydroreal4.ge.3) iutherm = irho+1
      call guess_labels(ncolumns,iamvec,label,labelvec,istartmhd,istart_extra_real4,&
           nmhd,nhydroreal4,ndimV,irho,iBfirst,ivx,iutherm,idivB,iJfirst,&
           iradenergy,icv,udist,utime,units,unitslabel)
@@ -2227,6 +2309,28 @@ subroutine set_labels
      iamvec(icurlvxcol:icurlvzcol) = icurlvxcol
      labelvec(icurlvxcol:icurlvzcol) = 'curl v'
   endif
+  if (idustfracsum.gt.0) then
+     ! Make N dustfrac labels
+     do i = idustfracsum+1,idustfrac
+        write(dustfrac_string,'(I10)') i-idustfracsum
+        write(dustfrac_string,'(A)') 'dustfrac'//trim(adjustl(dustfrac_string))
+        label(i) = dustfrac_string
+     enddo
+  endif
+  if (ideltavsum.gt.0) then
+     ! Modify the deltavsum labels to have vector subscripts
+     do j=1,ndimV
+        label(ideltavsum+j-1) = 'deltavsum'//'_'//labelcoord(j,1)
+     enddo
+     ! Make N deltav labels with vector subscripts
+     do i = ideltavsum+ndimV,ideltav,ndimV
+        write(deltav_string,'(I10)') (i-ideltavsum)/ndimV
+        write(deltav_string,'(A)') 'deltav'//trim(adjustl(deltav_string))
+        do j=1,ndimV
+           label(i+j-1) = trim(deltav_string)//'_'//labelcoord(j,1)
+        enddo
+     enddo 
+  endif
   !
   !--set labels for vector quantities
   !
@@ -2274,6 +2378,14 @@ subroutine set_labels
       units(ivx:ivx+ndimV-1) = udist/utime
       unitslabel(ivx:ivx+ndimV-1) = ' [cm/s]'
    endif
+   if (ideltavsum.gt.0) then
+      units(ideltavsum:ideltav+ndimV-1) = udist/utime
+      unitslabel(ideltavsum:ideltav+ndimV-1) = ' [cm/s]'
+   endif
+   if (ideltav.gt.0) then
+      units(ideltav:ideltav+ndimV-1) = udist/utime
+      unitslabel(ideltav:ideltav+ndimV-1) = ' [cm/s]'
+   endif
    if (iutherm.gt.0) then
       units(iutherm) = (udist/utime)**2
       unitslabel(iutherm) = ' [erg/g]'
